@@ -29,10 +29,12 @@ use crate::songtag::lrc::Lyric;
 use anyhow::{anyhow, bail, Result};
 use humantime::{format_duration, FormattedDuration};
 use id3::frame::{Lyrics, Picture, PictureType};
+use lofty::{Accessor, ItemKey, Probe, Tag};
 use metaflac::Tag as FlacTag;
 use mp4ameta::{Img, ImgFmt};
 use ogg_picture::{MimeType, PictureType as OggPictureType};
 use ogg_reader_writer::{replace_comment_header, CommentHeader, VorbisComments};
+use std::convert::From;
 use std::ffi::OsStr;
 use std::fs::{rename, File};
 use std::io::{Cursor, Read};
@@ -40,6 +42,17 @@ use std::path::Path;
 use std::str::FromStr;
 use std::time::Duration;
 
+#[derive(Clone)]
+pub enum AudioFormat {
+    MP3,
+    Wav,
+    Ogg,
+    Opus,
+    Flac,
+    M4A,
+    Unsupported,
+}
+
 #[derive(Clone)]
 pub struct Song {
     /// Artist of the song
@@ -61,6 +74,7 @@ pub struct Song {
     lyric_selected_index: usize,
     parsed_lyric: Option<Lyric>,
     picture: Option<Picture>,
+    format: AudioFormat,
 }
 
 impl Song {
@@ -227,13 +241,14 @@ impl Song {
     // }
 
     pub fn save_tag(&mut self) -> Result<()> {
-        match self.ext() {
-            Some("mp3") => self.save_mp3_tag()?,
-            Some("m4a") => self.save_m4a_tag()?,
-            Some("flac") => self.save_flac_tag()?,
-            Some("ogg") => self.save_ogg_tag()?,
-            Some("wav") => self.save_wav_tag()?,
-            _ => return Ok(()),
+        match self.format {
+            AudioFormat::MP3 => self.save_mp3_tag()?,
+            AudioFormat::M4A => self.save_m4a_tag()?,
+            AudioFormat::Flac => self.save_flac_tag()?,
+            AudioFormat::Ogg => self.save_ogg_tag()?,
+            AudioFormat::Opus => self.save_opus_tag()?,
+            AudioFormat::Wav => self.save_wav_tag()?,
+            AudioFormat::Unsupported => return Ok(()),
         }
 
         self.rename_by_tag()?;
@@ -391,7 +406,7 @@ impl Song {
         if let Some(p) = &self.picture {
             let mime_type = match p.mime_type.as_str() {
                 "image/bmp" => MimeType::Bmp,
-                "image/Png" => MimeType::Png,
+                "image/png" => MimeType::Png,
                 "image/jpeg" | &_ => MimeType::Jpeg,
             };
             let picture_ogg = ogg_picture::OggPicture::new(
@@ -415,6 +430,59 @@ impl Song {
         Ok(())
     }
 
+    #[allow(clippy::option_if_let_else)]
+    fn save_opus_tag(&self) -> Result<()> {
+        //open files
+        let file = self.file().ok_or_else(|| anyhow!("no file found"))?;
+
+        let p = Path::new(file);
+        let mut tagged_file = Probe::open(file)?.read(false)?;
+
+        let tag = match tagged_file.primary_tag_mut() {
+            Some(primary_tag) => primary_tag,
+            None => {
+                if let Some(first_tag) = tagged_file.first_tag_mut() {
+                    first_tag
+                } else {
+                    let tag_type = tagged_file.primary_tag_type();
+
+                    eprintln!(
+                        "WARN: No tags found, creating a new tag of type `{:?}`",
+                        tag_type
+                    );
+                    tagged_file.insert_tag(Tag::new(tag_type));
+
+                    tagged_file.primary_tag_mut().unwrap()
+                }
+            }
+        };
+
+        tag.set_title(self.title().unwrap_or("Unknown title").to_string());
+        tag.set_artist(self.artist().unwrap_or("Unknown artist").to_string());
+        tag.set_album(self.album().unwrap_or("").to_string());
+
+        if let Some(p) = &self.picture {
+            let mime_type = lofty::MimeType::from_str(p.mime_type.as_str());
+            let picture_opus = lofty::Picture::new_unchecked(
+                lofty::PictureType::CoverFront,
+                mime_type,
+                Some(p.description.clone()),
+                p.data.clone(),
+            );
+            tag.remove_picture_type(lofty::PictureType::CoverFront);
+            tag.push_picture(picture_opus);
+        }
+        if !self.lyric_frames.is_empty() {
+            let lyric_frames = self.lyric_frames.clone();
+            for l in lyric_frames {
+                tag.insert_text(ItemKey::Lyrics, l.text);
+            }
+        }
+        tag.save_to_path(p)
+            .map_err(|e| anyhow!("save opus file by lofty error: {}", e))?;
+        Ok(())
+    }
+
     fn save_wav_tag(&self) -> Result<()> {
         let mut id3_tag = id3::Tag::default();
         if let Some(file) = self.file() {
@@ -578,6 +646,7 @@ impl Song {
             lyric_selected_index: 0,
             parsed_lyric,
             picture,
+            format: AudioFormat::MP3,
         }
     }
 
@@ -661,6 +730,7 @@ impl Song {
             lyric_selected_index: 0,
             parsed_lyric,
             picture,
+            format: AudioFormat::Wav,
         }
     }
 
@@ -736,6 +806,7 @@ impl Song {
             lyric_selected_index: 0,
             parsed_lyric,
             picture,
+            format: AudioFormat::M4A,
         }
     }
 
@@ -842,6 +913,7 @@ impl Song {
             lyric_selected_index: 0,
             parsed_lyric,
             picture,
+            format: AudioFormat::Flac,
         }
     }
     fn from_ogg(s: &str) -> Self {
@@ -880,13 +952,7 @@ impl Song {
         let mut picture: Option<Picture> = None;
         if let Ok(picture_decoded) = base64::decode(picture_encoded) {
             if let Ok(p) = ogg_picture::OggPicture::from_apic_bytes(&picture_decoded) {
-                let mime_type = match p.mime_type {
-                    MimeType::Jpeg => "image/jpeg".to_string(),
-                    MimeType::Png => "image/png".to_string(),
-                    MimeType::Bmp => "image/bmp".to_string(),
-                    MimeType::Gif => "image/gif".to_string(),
-                    MimeType::Tiff => "image/tiff".to_string(),
-                };
+                let mime_type = String::from(p.mime_type);
                 let p_id3 = Picture {
                     mime_type,
                     picture_type: PictureType::CoverFront,
@@ -929,6 +995,98 @@ impl Song {
             lyric_selected_index: 0,
             parsed_lyric,
             picture,
+            format: AudioFormat::Ogg,
+        }
+    }
+    #[allow(clippy::option_if_let_else)]
+    fn from_opus(s: &str) -> Self {
+        let p: &Path = Path::new(s);
+        let ext = p.extension().and_then(OsStr::to_str);
+
+        let name = p
+            .file_name()
+            .and_then(OsStr::to_str)
+            .map(std::string::ToString::to_string);
+        let file = Some(String::from(s));
+
+        let mut tagged_file = Probe::open(p)
+            .expect("Error: Bad path provided!")
+            .read(true)
+            .expect("Error: Failed to read file!");
+
+        let tag = match tagged_file.primary_tag_mut() {
+            Some(primary_tag) => primary_tag,
+            None => {
+                if let Some(first_tag) = tagged_file.first_tag_mut() {
+                    first_tag
+                } else {
+                    let tag_type = tagged_file.primary_tag_type();
+
+                    eprintln!(
+                        "WARN: No tags found, creating a new tag of type `{:?}`",
+                        tag_type
+                    );
+                    tagged_file.insert_tag(Tag::new(tag_type));
+
+                    tagged_file.primary_tag_mut().unwrap()
+                }
+            }
+        };
+
+        //get the title, album, and artist of the song
+        let title = tag.title().unwrap_or("Unknown title").to_string();
+        let album = tag.album().unwrap_or("").to_string();
+        let artist = tag.artist().unwrap_or("Unknown Artist").to_string();
+        let lyrics_text = tag.get_string(&ItemKey::Lyrics).unwrap_or("").to_string();
+
+        let picture_lofty = tag.pictures().first();
+        let mut picture: Option<Picture> = None;
+        if let Some(p) = picture_lofty {
+            let mime_type = p.mime_type().as_str().to_string();
+            let description = p.description().unwrap_or("some image").to_string();
+            let p_id3 = Picture {
+                mime_type,
+                picture_type: PictureType::CoverFront,
+                description,
+                data: p.data().to_vec(),
+            };
+            picture = Some(p_id3);
+        }
+        let mut lyric_frames: Vec<Lyrics> = Vec::new();
+        let mut parsed_lyric: Option<Lyric> = None;
+        if lyrics_text.len() > 10 {
+            let lyrics = Lyrics {
+                lang: "eng".to_string(),
+                description: "termusic".to_string(),
+                text: lyrics_text,
+            };
+            lyric_frames = vec![lyrics];
+            if let Some(l) = lyric_frames.get(0) {
+                parsed_lyric = match Lyric::from_str(&l.text) {
+                    Ok(l) => Some(l),
+                    Err(_) => None,
+                }
+            }
+        }
+
+        //get the song duration
+        // let duration = GStreamer::duration(s).into();
+        let properties = tagged_file.properties();
+        let duration = properties.duration();
+
+        Self {
+            artist: Some(artist),
+            album: Some(album),
+            title: Some(title),
+            file,
+            duration,
+            name,
+            ext: ext.map(String::from),
+            lyric_frames,
+            lyric_selected_index: 0,
+            parsed_lyric,
+            picture,
+            format: AudioFormat::Opus,
         }
     }
 }
@@ -946,6 +1104,7 @@ impl FromStr for Song {
             Some("m4a") => Ok(Self::from_m4a(s)),
             Some("flac") => Ok(Self::from_flac(s)),
             Some("ogg") => Ok(Self::from_ogg(s)),
+            Some("opus") => Ok(Self::from_opus(s)),
             Some("wav") => Ok(Self::from_wav(s)),
             _ => {
                 let artist = Some(String::from("Not Support?"));
@@ -969,6 +1128,7 @@ impl FromStr for Song {
                     lyric_selected_index: 0,
                     parsed_lyric,
                     picture,
+                    format: AudioFormat::Unsupported,
                 })
             }
         }
