@@ -1,5 +1,6 @@
+use crate::ui::components::StyleColorSymbol;
 use crate::ui::model::MAX_DEPTH;
-use crate::ui::{Id, Model, Msg};
+use crate::ui::{Id, LIMsg, Model, Msg, TEMsg, YSMsg};
 use anyhow::{bail, Result};
 use if_chain::if_chain;
 use std::fs::{remove_dir_all, remove_file, rename};
@@ -8,7 +9,8 @@ use tui_realm_treeview::{Node, Tree, TreeView, TREE_CMD_CLOSE, TREE_CMD_OPEN, TR
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 use tuirealm::props::{Alignment, BorderType, Borders, TableBuilder, TextSpan};
-use tuirealm::tui::style::{Color, Style};
+// use tuirealm::tui::style::{Color, Style};
+use tuirealm::tui::style::Color;
 use tuirealm::{
     AttrValue, Attribute, Component, Event, MockComponent, NoUserEvent, State, StateValue,
 };
@@ -19,7 +21,11 @@ pub struct MusicLibrary {
 }
 
 impl MusicLibrary {
-    pub fn new(tree: &Tree, initial_node: Option<String>) -> Self {
+    pub fn new(
+        tree: &Tree,
+        initial_node: Option<String>,
+        color_mapping: &StyleColorSymbol,
+    ) -> Self {
         // Preserve initial node if exists
         let initial_node = match initial_node {
             Some(id) if tree.root().query(&id).is_some() => id,
@@ -27,18 +33,19 @@ impl MusicLibrary {
         };
         Self {
             component: TreeView::default()
-                .foreground(Color::Reset)
+                .background(color_mapping.library_background().unwrap_or(Color::Reset))
+                .foreground(color_mapping.library_foreground().unwrap_or(Color::Magenta))
                 .borders(
                     Borders::default()
-                        .color(Color::LightYellow)
+                        .color(color_mapping.library_border().unwrap_or(Color::Magenta))
                         .modifiers(BorderType::Rounded),
                 )
-                .inactive(Style::default().fg(Color::Gray))
+                // .inactive(Style::default().fg(Color::Gray))
                 .indent_size(2)
                 .scroll_step(6)
                 .title("Library", Alignment::Left)
-                .highlighted_color(Color::LightYellow)
-                .highlight_symbol("\u{1f984}")
+                .highlighted_color(color_mapping.library_highlight().unwrap_or(Color::Yellow))
+                .highlight_symbol(&color_mapping.library_highlight_symbol)
                 .preserve_state(true)
                 // .highlight_symbol("🦄")
                 .with_tree(tree.clone())
@@ -48,6 +55,7 @@ impl MusicLibrary {
 }
 
 impl Component<Msg, NoUserEvent> for MusicLibrary {
+    #[allow(clippy::too_many_lines)]
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let result = match ev {
             Event::Keyboard(KeyEvent {
@@ -63,7 +71,9 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
                 if p.is_dir() {
                     self.perform(Cmd::Custom(TREE_CMD_OPEN))
                 } else {
-                    return Some(Msg::PlaylistAdd(current_node.to_string()));
+                    return Some(Msg::Playlist(crate::ui::PLMsg::Add(
+                        current_node.to_string(),
+                    )));
                 }
             }
             Event::Keyboard(KeyEvent {
@@ -73,7 +83,9 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
                 let current_node = self.component.tree_state().selected().unwrap();
                 let p: &Path = Path::new(current_node);
                 if p.is_dir() {
-                    return Some(Msg::PlaylistAdd(current_node.to_string()));
+                    return Some(Msg::Playlist(crate::ui::PLMsg::Add(
+                        current_node.to_string(),
+                    )));
                 }
                 return None;
             }
@@ -108,11 +120,11 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             Event::Keyboard(KeyEvent {
                 code: Key::Backspace,
                 modifiers: KeyModifiers::NONE,
-            }) => return Some(Msg::LibraryTreeGoToUpperDir),
+            }) => return Some(Msg::Library(LIMsg::TreeGoToUpperDir)),
             Event::Keyboard(KeyEvent {
                 code: Key::Tab,
                 modifiers: KeyModifiers::NONE,
-            }) => return Some(Msg::LibraryTreeBlur),
+            }) => return Some(Msg::Library(LIMsg::TreeBlur)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('d'),
                 ..
@@ -120,32 +132,34 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             Event::Keyboard(KeyEvent {
                 code: Key::Char('y'),
                 ..
-            }) => return Some(Msg::LibraryYank),
+            }) => return Some(Msg::Library(LIMsg::Yank)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('p'),
                 ..
-            }) => return Some(Msg::LibraryPaste),
+            }) => return Some(Msg::Library(LIMsg::Paste)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('/'),
                 ..
-            }) => return Some(Msg::GeneralSearchPopupShowLibrary),
+            }) => return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowLibrary)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('s'),
                 modifiers: KeyModifiers::NONE,
-            }) => return Some(Msg::YoutubeSearchInputPopupShow),
+            }) => return Some(Msg::YoutubeSearch(YSMsg::InputPopupShow)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('t'),
                 modifiers: KeyModifiers::NONE,
             }) => {
                 let current_node = self.component.tree_state().selected().unwrap();
-                return Some(Msg::TagEditorRun(current_node.to_string()));
+                return Some(Msg::TagEditor(TEMsg::TagEditorRun(
+                    current_node.to_string(),
+                )));
             }
 
             _ => return None,
         };
         match result {
             CmdResult::Submit(State::One(StateValue::String(node))) => {
-                Some(Msg::LibraryTreeExtendDir(node))
+                Some(Msg::Library(LIMsg::TreeExtendDir(node)))
             }
             _ => Some(Msg::None),
         }
@@ -162,28 +176,6 @@ impl Model {
         self.path.parent().map(std::path::Path::to_path_buf)
     }
 
-    // pub fn extend_dir(&mut self, id: &str, p: &Path, depth: usize) {
-    //     if let Some(node) = self.tree.root_mut().query_mut(&String::from(id)) {
-    //         if depth > 0 && p.is_dir() {
-    //             // Clear node
-    //             node.clear();
-    //             // Scan dir
-    //             if let Ok(paths) = std::fs::read_dir(p) {
-    //                 let mut paths: Vec<_> = paths.filter_map(std::result::Result::ok).collect();
-
-    //                 paths.sort_by_cached_key(|k| {
-    //                     get_pin_yin(&k.file_name().to_string_lossy().to_string())
-    //                 });
-    //                 for p in paths {
-    //                     if !p.path().is_dir() {
-    //                         node.add_child(Self::dir_tree(p.path().as_path(), depth - 1));
-    //                     }
-    //                 }
-    //             }
-    //         }
-    //     }
-    // }
-
     pub fn library_dir_tree(p: &Path, depth: usize) -> Node {
         let name: String = match p.file_name() {
             None => "/".to_string(),
@@ -224,12 +216,6 @@ impl Model {
     }
 
     pub fn library_sync(&mut self, node: Option<&str>) {
-        // self.tree = Tree::new(Self::dir_tree(self.path.as_ref(), 3));
-        // assert!(self
-        //     .app
-        //     .attr(&Id::Library, Attribute::, AttrValue::Tr(table),)
-        //     .is_ok());
-
         self.library_reload_tree();
         if let Some(n) = node {
             assert!(self
@@ -250,16 +236,30 @@ impl Model {
             _ => None,
         };
         // Remount tree
+        // keep focus
+        let mut focus = false;
+        if let Ok(f) = self.app.query(&Id::Library, Attribute::Focus) {
+            if Some(AttrValue::Flag(true)) == f {
+                focus = true;
+            }
+        }
+
         assert!(self.app.umount(&Id::Library).is_ok());
         assert!(self
             .app
             .mount(
                 Id::Library,
-                Box::new(MusicLibrary::new(&self.tree.clone(), current_node)),
+                Box::new(MusicLibrary::new(
+                    &self.tree.clone(),
+                    current_node,
+                    &self.config.style_color_symbol,
+                ),),
                 Vec::new()
             )
             .is_ok());
-        assert!(self.app.active(&Id::Library).is_ok());
+        if focus {
+            assert!(self.app.active(&Id::Library).is_ok());
+        }
     }
 
     pub fn library_stepinto(&mut self, node_id: &str) {
@@ -382,7 +382,7 @@ impl Model {
         }
         let table = table.build();
 
-        self.general_search_update_library(table);
+        self.general_search_update_show(table);
     }
 }
 
