@@ -2,10 +2,10 @@
 use crate::{
     config::get_app_config_path,
     song::Song,
-    ui::{components::table_playlist::TABLE_INITIAL_INDEX, Id, Model, Msg},
+    ui::{GSMsg, Id, Model, Msg, PLMsg},
 };
 
-use super::Table;
+use crate::ui::components::StyleColorSymbol;
 use crate::ui::Loop;
 use anyhow::Result;
 use humantime::format_duration;
@@ -18,8 +18,9 @@ use std::path::Path;
 use std::str::FromStr;
 use std::thread;
 use std::time::Duration;
+use tui_realm_stdlib::Table;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
-use tuirealm::props::{Alignment, BorderType, TableBuilder, TextSpan};
+use tuirealm::props::{Alignment, BorderType, PropPayload, PropValue, TableBuilder, TextSpan};
 use tuirealm::{
     event::{Key, KeyEvent, KeyModifiers},
     AttrValue, Attribute, Component, Event, MockComponent, NoUserEvent, State, StateValue,
@@ -32,21 +33,26 @@ pub struct Playlist {
     component: Table,
 }
 
-impl Default for Playlist {
-    fn default() -> Self {
+impl Playlist {
+    pub fn new(color_mapping: &StyleColorSymbol) -> Self {
         Self {
             component: Table::default()
                 .borders(
                     Borders::default()
-                        .modifiers(BorderType::Thick)
-                        .color(Color::Blue),
+                        .modifiers(BorderType::Rounded)
+                        .color(color_mapping.playlist_border().unwrap_or(Color::Blue)),
                 )
-                // .foreground(Color::Yellow)
-                .background(Color::Reset)
+                .background(color_mapping.playlist_background().unwrap_or(Color::Reset))
+                .foreground(color_mapping.playlist_foreground().unwrap_or(Color::Yellow))
                 .title("Playlist", Alignment::Left)
                 .scroll(true)
-                .highlighted_color(Color::LightBlue)
-                .highlighted_str("\u{1f680}")
+                .highlighted_color(
+                    color_mapping
+                        .playlist_highlight()
+                        .unwrap_or(Color::LightBlue),
+                )
+                .highlighted_str(&color_mapping.playlist_highlight_symbol)
+                // .highlighted_str("\u{1f680}")
                 // .highlighted_str("🚀")
                 .rewind(true)
                 .step(4)
@@ -95,49 +101,46 @@ impl Component<Msg, NoUserEvent> for Playlist {
                 },
             ) => self.perform(Cmd::GoTo(Position::End)),
             Event::Keyboard(KeyEvent { code: Key::Tab, .. }) => {
-                return Some(Msg::PlaylistTableBlur)
+                return Some(Msg::Playlist(PLMsg::TableBlur))
             }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('d'),
                 ..
             }) => match self.component.state() {
                 State::One(StateValue::Usize(index_selected)) => {
-                    return Some(Msg::PlaylistDelete(index_selected))
+                    return Some(Msg::Playlist(PLMsg::Delete(index_selected)))
                 }
                 _ => return Some(Msg::None),
             },
             Event::Keyboard(KeyEvent {
                 code: Key::Char('D'),
                 modifiers: KeyModifiers::SHIFT,
-            }) => return Some(Msg::PlaylistDeleteAll),
+            }) => return Some(Msg::Playlist(PLMsg::DeleteAll)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('s'),
                 ..
-            }) => return Some(Msg::PlaylistShuffle),
+            }) => return Some(Msg::Playlist(PLMsg::Shuffle)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('m'),
                 ..
-            }) => return Some(Msg::PlaylistLoopModeCycle),
+            }) => return Some(Msg::Playlist(PLMsg::LoopModeCycle)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('l'),
                 ..
             }) => {
                 if let State::One(StateValue::Usize(index)) = self.state() {
-                    return Some(Msg::PlaylistPlaySelected(index));
+                    return Some(Msg::Playlist(PLMsg::PlaySelected(index)));
                 }
                 CmdResult::None
             }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('a'),
                 ..
-            }) => return Some(Msg::PlaylistAddFront),
+            }) => return Some(Msg::Playlist(PLMsg::AddFront)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('/'),
                 ..
-            }) => return Some(Msg::GeneralSearchPopupShowPlaylist),
-            // Event::User(UserEvent::GotoTableIndex(index)) => {
-            //     self.perform(Cmd::GoTo(Position::At(index)))
-            // }
+            }) => return Some(Msg::GeneralSearch(GSMsg::PopupShowPlaylist)),
             _ => CmdResult::None,
         };
         // match cmd_result {
@@ -151,6 +154,18 @@ impl Component<Msg, NoUserEvent> for Playlist {
 }
 
 impl Model {
+    pub fn playlist_reload(&mut self) {
+        assert!(self.app.umount(&Id::Playlist).is_ok());
+        assert!(self
+            .app
+            .mount(
+                Id::Playlist,
+                Box::new(Playlist::new(&self.config.style_color_symbol)),
+                Vec::new()
+            )
+            .is_ok());
+        self.playlist_sync();
+    }
     pub fn playlist_add_item(
         &mut self,
         current_node: &str,
@@ -236,7 +251,7 @@ impl Model {
     pub fn playlist_empty(&mut self) {
         self.playlist_items.clear();
         self.playlist_sync();
-        self.app.active(&Id::Library).ok();
+        // self.app.active(&Id::Library).ok();
     }
 
     pub fn playlist_save(&mut self) -> Result<()> {
@@ -373,36 +388,51 @@ impl Model {
         let mut idx = 0;
         let search = format!("*{}*", input.to_lowercase());
         for record in &self.playlist_items {
-            // for record in all_items.into_iter().filter_map(std::result::Result::ok) {
-            if let Some(file_name) = record.name() {
-                if wildmatch::WildMatch::new(&search).matches(&file_name.to_string().to_lowercase())
-                {
-                    if idx > 0 {
-                        table.add_row();
-                    }
-                    idx += 1;
-                    table
-                        .add_col(TextSpan::new(idx.to_string()))
-                        .add_col(TextSpan::new(file_name.to_string()));
+            let artist = record.artist().unwrap_or("Unknown artist");
+            let title = record.title().unwrap_or("Unknown title");
+            if wildmatch::WildMatch::new(&search).matches(&artist.to_lowercase())
+                | wildmatch::WildMatch::new(&search).matches(&title.to_lowercase())
+            {
+                if idx > 0 {
+                    table.add_row();
                 }
+
+                let duration = record.duration_formatted().to_string();
+                let duration_string = format!("[{:^6.6}]", duration);
+
+                let noname_string = "No Name".to_string();
+                let name = record.name().unwrap_or(&noname_string);
+                let artist = record.artist().unwrap_or(name);
+                let title = record.title().unwrap_or("Unknown Title");
+                let file_name = record.file().unwrap_or("no file");
+
+                table
+                    .add_col(TextSpan::new(duration_string.as_str()))
+                    .add_col(TextSpan::new(artist).fg(tuirealm::tui::style::Color::LightYellow))
+                    .add_col(TextSpan::new(title).bold())
+                    .add_col(TextSpan::new(file_name));
+                // .add_col(TextSpan::new(record.album().unwrap_or("Unknown Album")));
+                idx += 1;
             }
         }
+        if self.playlist_items.is_empty() {
+            table.add_col(TextSpan::from("0"));
+            table.add_col(TextSpan::from("empty playlist"));
+            table.add_col(TextSpan::from(""));
+        }
         let table = table.build();
 
-        self.general_search_update_library(table);
+        self.general_search_update_show(table);
     }
 
-    #[allow(clippy::cast_possible_wrap)]
-    pub fn playlist_locate(&mut self, index: Option<usize>) {
-        if let Some(i) = index {
-            assert!(self
-                .app
-                .attr(
-                    &Id::Playlist,
-                    Attribute::Custom(TABLE_INITIAL_INDEX),
-                    AttrValue::Number(i as isize),
-                )
-                .is_ok());
-        }
+    pub fn playlist_locate(&mut self, index: usize) {
+        assert!(self
+            .app
+            .attr(
+                &Id::Playlist,
+                Attribute::Value,
+                AttrValue::Payload(PropPayload::One(PropValue::Usize(index))),
+            )
+            .is_ok());
     }
 }
