@@ -185,6 +185,9 @@ impl Default for HelpPopup {
                         .add_col(TextSpan::new("<+,=/-,_>").bold().fg(Color::Cyan))
                         .add_col(TextSpan::from("Increase/Decrease volume"))
                         .add_row()
+                        .add_col(TextSpan::new("<C>").bold().fg(Color::Cyan))
+                        .add_col(TextSpan::from("Open Color Editor(switch themes)"))
+                        .add_row()
                         .add_col(TextSpan::new("Library").bold().fg(Color::LightYellow))
                         .add_row()
                         .add_col(TextSpan::new("<l/L>").bold().fg(Color::Cyan))
@@ -229,6 +232,9 @@ impl Default for HelpPopup {
                         .add_col(TextSpan::from(
                             "Add a song to the front of playlist or back",
                         ))
+                        .add_row()
+                        .add_col(TextSpan::new("</>").bold().fg(Color::Cyan))
+                        .add_col(TextSpan::from("Search in playlist"))
                         .build(),
                 ),
         }
