@@ -6,6 +6,7 @@ use if_chain::if_chain;
 use std::time::Duration;
 use tui_realm_stdlib::ProgressBar;
 // use tuirealm::command::CmdResult;
+use crate::ui::components::StyleColorSymbol;
 use tuirealm::props::{Alignment, BorderType, Borders, Color, PropPayload, PropValue};
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, NoUserEvent};
 
@@ -14,16 +15,29 @@ pub struct Progress {
     component: ProgressBar,
 }
 
-impl Default for Progress {
-    fn default() -> Self {
+impl Progress {
+    pub fn new(style_color_symbol: &StyleColorSymbol) -> Self {
         Self {
             component: ProgressBar::default()
                 .borders(
                     Borders::default()
-                        .color(Color::LightMagenta)
+                        .color(
+                            style_color_symbol
+                                .progress_border()
+                                .unwrap_or(Color::LightMagenta),
+                        )
                         .modifiers(BorderType::Rounded),
                 )
-                .foreground(Color::DarkGray)
+                .background(
+                    style_color_symbol
+                        .progress_background()
+                        .unwrap_or(Color::Reset),
+                )
+                .foreground(
+                    style_color_symbol
+                        .progress_foreground()
+                        .unwrap_or(Color::Yellow),
+                )
                 .label("Song Name")
                 .title("Playing", Alignment::Center)
                 .progress(0.0),
@@ -33,14 +47,24 @@ impl Default for Progress {
 
 impl Component<Msg, NoUserEvent> for Progress {
     fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
-        // let _drop = match ev {
-        //     _ => CmdResult::None,
-        // };
         Some(Msg::None)
     }
 }
 
 impl Model {
+    pub fn progress_reload(&mut self) {
+        assert!(self.app.umount(&Id::Progress).is_ok());
+        assert!(self
+            .app
+            .mount(
+                Id::Progress,
+                Box::new(Progress::new(&self.config.style_color_symbol)),
+                Vec::new()
+            )
+            .is_ok());
+        self.progress_update_title();
+    }
+
     pub fn progress_update_title(&mut self) {
         if_chain! {
             if let Some(song) = &self.current_song;
@@ -62,7 +86,7 @@ impl Model {
 
     pub fn progress_update(&mut self) {
         if let Ok((new_prog, time_pos, duration)) = self.player.get_progress() {
-            if (new_prog, time_pos, duration) == (0.9, 0, 119) {
+            if (new_prog, time_pos, duration) == (0.0, 0, 119) {
                 return;
             }
 
@@ -82,6 +106,7 @@ impl Model {
                 return;
             }
             self.time_pos = time_pos;
+            let new_prog = new_prog / 100.0;
             self.app
                 .attr(
                     &Id::Progress,
