@@ -34,6 +34,7 @@ use crate::config::Termusic;
 use crate::songtag::SongTag;
 use model::Model;
 // Let's define the messages handled by our app. NOTE: it must derive `PartialEq`
+use crate::ui::components::ColorConfig;
 use serde::{Deserialize, Serialize};
 use std::time::Duration;
 use tuirealm::application::PollStrategy;
@@ -46,49 +47,81 @@ const FORCED_REDRAW_INTERVAL: Duration = Duration::from_millis(1000);
 #[derive(Debug, PartialEq)]
 pub enum Msg {
     // AppClose,
+    ColorEditor(CEMsg),
     DeleteConfirmCloseCancel,
     DeleteConfirmCloseOk,
     DeleteConfirmShow,
     ErrorPopupClose,
+    GeneralSearch(GSMsg),
     HelpPopupShow,
     HelpPopupClose,
-    LibraryTreeExtendDir(String),
-    LibraryTreeGoToUpperDir,
-    LibraryTreeBlur,
-    LibraryYank,
-    LibraryPaste,
-    GeneralSearchPopupShowLibrary,
-    GeneralSearchPopupShowPlaylist,
-    GeneralSearchPopupCloseCancel,
-    GeneralSearchInputBlur,
-    GeneralSearchPopupUpdateLibrary(String),
-    GeneralSearchPopupUpdatePlaylist(String),
-    GeneralSearchTableBlur,
-    GeneralSearchPopupCloseLibraryAddPlaylist,
-    GeneralSearchPopupCloseOkLibraryLocate,
-    GeneralSearchPopupClosePlaylistPlaySelected,
-    GeneralSearchPopupCloseOkPlaylistLocate,
+    Library(LIMsg),
     LyricCycle,
     LyricAdjustDelay(i64),
     PlayerTogglePause,
     PlayerVolumeUp,
     PlayerVolumeDown,
     PlayerSeek(isize),
-    PlaylistAddFront,
-    PlaylistNextSong,
-    PlaylistPrevSong,
-    PlaylistTableBlur,
-    PlaylistAdd(String),
-    PlaylistDelete(usize),
-    PlaylistDeleteAll,
-    PlaylistLoopModeCycle,
-    PlaylistPlaySelected(usize),
-    PlaylistShuffle,
+    Playlist(PLMsg),
     QuitPopupCloseCancel,
     QuitPopupCloseOk,
     QuitPopupShow,
+    TagEditor(TEMsg),
+    UpdatePhoto,
+    YoutubeSearch(YSMsg),
+    None,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum LIMsg {
+    TreeExtendDir(String),
+    TreeGoToUpperDir,
+    TreeBlur,
+    Yank,
+    Paste,
+}
+#[derive(Debug, PartialEq)]
+pub enum PLMsg {
+    AddFront,
+    NextSong,
+    PrevSong,
+    TableBlur,
+    Add(String),
+    Delete(usize),
+    DeleteAll,
+    LoopModeCycle,
+    PlaySelected(usize),
+    Shuffle,
+}
+#[derive(Debug, PartialEq)]
+pub enum GSMsg {
+    PopupShowLibrary,
+    PopupShowPlaylist,
+    PopupCloseCancel,
+    InputBlur,
+    PopupUpdateLibrary(String),
+    PopupUpdatePlaylist(String),
+    TableBlur,
+    PopupCloseLibraryAddPlaylist,
+    PopupCloseOkLibraryLocate,
+    PopupClosePlaylistPlaySelected,
+    PopupCloseOkPlaylistLocate,
+}
+
+#[derive(Debug, PartialEq)]
+pub enum YSMsg {
+    InputPopupShow,
+    InputPopupCloseCancel,
+    InputPopupCloseOk(String),
+    TablePopupNext,
+    TablePopupPrevious,
+    TablePopupCloseCancel,
+    TablePopupCloseOk(usize),
+}
+#[derive(Debug, PartialEq)]
+pub enum TEMsg {
     TagEditorRun(String),
-    TagEditorBlur(Option<String>),
+    TagEditorClose(Option<String>),
     TECounterDeleteBlur,
     TECounterDeleteOk,
     TEDownload(usize),
@@ -104,19 +137,41 @@ pub enum Msg {
     TESelectLyricOk(usize),
     TETableLyricOptionsBlur,
     TETextareaLyricBlur,
-    YoutubeSearchInputPopupShow,
-    YoutubeSearchInputPopupCloseCancel,
-    YoutubeSearchInputPopupCloseOk(String),
-    YoutubeSearchTablePopupNext,
-    YoutubeSearchTablePopupPrevious,
-    YoutubeSearchTablePopupCloseCancel,
-    YoutubeSearchTablePopupCloseOk(usize),
-    None,
 }
-
+// #[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, PartialEq)]
+pub enum CEMsg {
+    ColorChanged(IdColorEditor, ColorConfig),
+    SymbolChanged(IdColorEditor, String),
+    ColorEditorShow,
+    ColorEditorCloseCancel,
+    ColorEditorCloseOk,
+    HelpPopupShow,
+    HelpPopupClose,
+    LibraryForegroundBlur,
+    LibraryBackgroundBlur,
+    LibraryBorderBlur,
+    LibraryHighlightBlur,
+    LibraryHighlightSymbolBlur,
+    PlaylistForegroundBlur,
+    PlaylistBackgroundBlur,
+    PlaylistBorderBlur,
+    PlaylistHighlightBlur,
+    PlaylistHighlightSymbolBlur,
+    ProgressForegroundBlur,
+    ProgressBackgroundBlur,
+    ProgressBorderBlur,
+    LyricForegroundBlur,
+    LyricBackgroundBlur,
+    LyricBorderBlur,
+    ThemeSelectBlur,
+    ThemeSelectLoad(usize),
+    ColorEditorOkBlur,
+}
 // Let's define the component ids for our application
 #[derive(Debug, Eq, PartialEq, Clone, Hash)]
 pub enum Id {
+    ColorEditor(IdColorEditor),
     DeleteConfirmRadioPopup,
     DeleteConfirmInputPopup,
     ErrorPopup,
@@ -131,6 +186,13 @@ pub enum Id {
     Playlist,
     Progress,
     QuitPopup,
+    TagEditor(IdTagEditor),
+    YoutubeSearchInputPopup,
+    YoutubeSearchTablePopup,
+}
+
+#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+pub enum IdTagEditor {
     TECounterDelete,
     TEHelpPopup,
     TELabelHint,
@@ -140,8 +202,33 @@ pub enum Id {
     TESelectLyric,
     TETableLyricOptions,
     TETextareaLyric,
-    YoutubeSearchInputPopup,
-    YoutubeSearchTablePopup,
+}
+#[derive(Debug, Eq, PartialEq, Clone, Hash)]
+pub enum IdColorEditor {
+    RadioOk,
+    LabelHint,
+    ThemeSelect,
+    HelpPopup,
+    LibraryLabel,
+    LibraryForeground,
+    LibraryBackground,
+    LibraryBorder,
+    LibraryHighlight,
+    LibraryHighlightSymbol,
+    PlaylistLabel,
+    PlaylistForeground,
+    PlaylistBackground,
+    PlaylistBorder,
+    PlaylistHighlight,
+    PlaylistHighlightSymbol,
+    ProgressLabel,
+    ProgressForeground,
+    ProgressBackground,
+    ProgressBorder,
+    LyricLabel,
+    LyricForeground,
+    LyricBackground,
+    LyricBorder,
 }
 
 #[derive(Clone, Copy)]
@@ -195,7 +282,8 @@ impl UI {
     pub fn new(config: &Termusic) -> Self {
         let mut model = Model::new(config);
         model.init_config();
-        model.library_reload_tree();
+        // model.library_reload_tree();
+        // assert!(model.app.active(&Id::Library).is_ok());
         Self { model }
     }
     /// ### run
@@ -213,8 +301,8 @@ impl UI {
             self.model.te_update_lyric_options();
             self.model.update_playlist_items();
             self.model.update_components();
-            self.model.progress_update();
             self.model.update_lyric();
+            self.model.progress_update();
 
             if progress_interval == 0 {
                 self.model.run();
